////////////////////////////////////////////////////////////////////////////////
/** @brief タイマを使ってPWM波を出力するための関数
@date 2012/3/10
*///////////////////////////////////////////////////////////////////////////////
#include <stm32f10x.h>
#include "config_PWM.h"

#define f_PWM_motor 200000 // PWM frequency [Hz]
uint16_t TIM_Period_PWM_motor;
////////////////////////////////////////////////////////////////////////////////
/*
使用するペリフェラル
- TIM8
使用するポート
- PC6 (TIM8_OC1)
- PC7 (TIM8_OC2)
- PC8 (TIM8_OC3)
- PC9 (TIM8_OC4)
*///////////////////////////////////////////////////////////////////////////////
#define RCC_ClockCmd_TIMx_PWM_motor(x)  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM8, x)
#define RCC_ClockCmd_GPIOx_PWM_motor(x) RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, x)
#define GPIOx_PWM_motor                 GPIOC
#define GPIO_Pin_x_PWM_motor1           GPIO_Pin_6
#define GPIO_Pin_x_PWM_motor2           GPIO_Pin_7
#define GPIO_Pin_x_PWM_motor3           GPIO_Pin_8
#define GPIO_Pin_x_PWM_motor4           GPIO_Pin_9
#define PCLKx_Frequency_PWM_motor       PCLK2_Frequency
#define TIMx_PWM_motor                  TIM8
#define TIM_OCMode_PWMx_PWM_motor       TIM_OCMode_PWM1 // PWM1: on-->off, PWM2: off-->on
#define TIM_SetCompareX_motorA_forward  TIM_SetCompare1
#define TIM_SetCompareX_motorA_backward TIM_SetCompare2
#define TIM_SetCompareX_motorB_forward  TIM_SetCompare4
#define TIM_SetCompareX_motorB_backward TIM_SetCompare3
////////////////////////////////////////////////////////////////////////////////
/** @brief PWM出力用にタイマを設定する
@details
4出力可能なタイマを使ってモータ駆動用PWM波を出力するための設定を行う．
想定しているPWM周波数は20-200kHzである．
(PWM_motor1, PWM_motor2)および(PWM_motor3, PWM_motor3)の組み合わせで
モータドライバICへPWM波を送り，2個のモータを駆動する．
@attention
使用するTIM, ポートはマクロで定義すること
@return なし
*///////////////////////////////////////////////////////////////////////////////
void TIM_config_for_PWM_motor()
{
	// PWM周波数からタイマのカウンタの設定値を計算する
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);
	if(RCC_Clocks.HCLK_Frequency == RCC_Clocks.PCLKx_Frequency_PWM_motor){
		TIM_Period_PWM_motor = RCC_Clocks.PCLKx_Frequency_PWM_motor / f_PWM_motor;
	}
	else{
		TIM_Period_PWM_motor = RCC_Clocks.PCLKx_Frequency_PWM_motor * 2 / f_PWM_motor;
	}

	// クロックを有効にする
	RCC_ClockCmd_TIMx_PWM_motor(ENABLE);
	RCC_ClockCmd_GPIOx_PWM_motor(ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	// PWM出力ポートの設定
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_InitStructure.GPIO_Pin  =  GPIO_Pin_x_PWM_motor1;
	GPIO_InitStructure.GPIO_Pin |=  GPIO_Pin_x_PWM_motor2;
	GPIO_InitStructure.GPIO_Pin |=  GPIO_Pin_x_PWM_motor3;
	GPIO_InitStructure.GPIO_Pin |=  GPIO_Pin_x_PWM_motor4;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOx_PWM_motor, &GPIO_InitStructure);

	// TIMの設定
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = TIM_Period_PWM_motor - 1; // 周期の設定
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseInit(TIMx_PWM_motor, &TIM_TimeBaseStructure);

	// PWM出力の設定
	TIM_OCInitTypeDef TIM_OCInitStructure;
	TIM_OCStructInit(&TIM_OCInitStructure);
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWMx_PWM_motor;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = 0; // initial duty
	TIM_OC1Init(TIMx_PWM_motor, &TIM_OCInitStructure);
	TIM_OC2Init(TIMx_PWM_motor, &TIM_OCInitStructure);
	TIM_OC3Init(TIMx_PWM_motor, &TIM_OCInitStructure);
	TIM_OC4Init(TIMx_PWM_motor, &TIM_OCInitStructure);
	TIM_CtrlPWMOutputs(TIMx_PWM_motor, ENABLE);
	
	// タイマ動作開始
	TIM_Cmd(TIMx_PWM_motor, ENABLE);
}


////////////////////////////////////////////////////////////////////////////////
/** @brief motorAを制御するPWM波のduty比を変更する
@param[in] duty duty比（千分率）
@return なし
*///////////////////////////////////////////////////////////////////////////////
void set_duty_PWM_motorA(int duty)
{
	int pulse;
	if(duty >= 0){
		if(duty > MAX_DUTY_PWM_motor){
			pulse = MAX_DUTY_PWM_motor;
		}
		else{
			pulse = (int)TIM_Period_PWM_motor * duty / 1000;
		}
		TIM_SetCompareX_motorA_forward(TIMx_PWM_motor, pulse);
		TIM_SetCompareX_motorA_backward(TIMx_PWM_motor, 0);
	}
	else{
		if(duty < -MAX_DUTY_PWM_motor){
			pulse = MAX_DUTY_PWM_motor;
		}
		else{
			pulse = - (int)TIM_Period_PWM_motor * duty / 1000;
		}
		TIM_SetCompareX_motorA_forward(TIMx_PWM_motor, 0);
		TIM_SetCompareX_motorA_backward(TIMx_PWM_motor, pulse);
	}
}


////////////////////////////////////////////////////////////////////////////////
/** @brief motorBを制御するPWM波のduty比を変更する
@param[in] duty duty比（千分率）
@return なし
*///////////////////////////////////////////////////////////////////////////////
void set_duty_PWM_motorB(int duty)
{
	int pulse;
	if(duty >= 0){
		if(duty > MAX_DUTY_PWM_motor){
			pulse = MAX_DUTY_PWM_motor;
		}
		else{
			pulse = (int)TIM_Period_PWM_motor * duty / 1000;
		}
		TIM_SetCompareX_motorB_forward(TIMx_PWM_motor, pulse);
		TIM_SetCompareX_motorB_backward(TIMx_PWM_motor, 0);
	}
	else{
		if(duty < -MAX_DUTY_PWM_motor){
			pulse = MAX_DUTY_PWM_motor;
		}
		else{
			pulse = - (int)TIM_Period_PWM_motor * duty / 1000;
		}
		TIM_SetCompareX_motorB_forward(TIMx_PWM_motor, 0);
		TIM_SetCompareX_motorB_backward(TIMx_PWM_motor, pulse);
	}
}

