////////////////////////////////////////////////////////////////////////////////
/** @brief クロックとFLASHの設定関数およびFLASHの消去と書き込み関数
@date 2012/3/10
*///////////////////////////////////////////////////////////////////////////////
#include <stm32f10x.h>
#include "config_clock_flash.h"

#define HSICalibrationValue 0x10 // 初期値は0x10, 0x0 - 0x1Fの間で調整
////////////////////////////////////////////////////////////////////////////////
/** @brief クロックとFLASHの設定を行う
@details
内蔵発振子を使用し，仕様上の最大値である以下の周波数に設定する
- SYSCLK = 64 MHz
- HCLK = 64 MHz (AHB clock)
- PCLK2 = 64 MHz (APB2 clock)
- PCLK1 = 32 MHz (APB1 clock)
- ADCCLK = 10.66... MHz
- APB2 timer clock = 64 MHz
- APB1 timer clock = 64 MHz
また，FLASHについて以下の項目を設定する
- wait state
- Instruction prefetchを有効にする
@note
APBxのprescalarが/1以外のとき，timerのクロックだけはx2される．
@return なし
*///////////////////////////////////////////////////////////////////////////////
void CLOCK_FLASH_config()
{
	// クロックの状態を初期値に戻す
	RCC_DeInit();

	// wait stateの設定
	FLASH_SetLatency(FLASH_Latency_2);
	// Enable Prefetch Buffer
	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

	// HSIのキャリブレーション値を設定する
	RCC_AdjustHSICalibrationValue(HSICalibrationValue);

	// PLLの設定をする (HSI / 2 * 16 = 64 MHz)
	RCC_PLLConfig(RCC_PLLSource_HSI_Div2, RCC_PLLMul_16);
	RCC_PLLCmd(ENABLE);
	while( RCC_GetFlagStatus(RCC_FLAG_PLLRDY) != SET);

	// HCLKの分周比を設定する
	RCC_HCLKConfig(RCC_SYSCLK_Div1);

	// PCLK2の分周比を設定する
	RCC_PCLK2Config(RCC_HCLK_Div1);

	// PLCK1の分周比を設定する
	RCC_PCLK1Config(RCC_HCLK_Div2);

	// ADCCLKの分周比を設定する
	RCC_ADCCLKConfig(RCC_PCLK2_Div6);

	// SYSCLKのクロックソースをPLLに切り替える
	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
	while(RCC_GetSYSCLKSource() != 0x08);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief FLASHの指定したセクターを消去する
@param[in] start_addr 消去エリア先頭アドレス
@param[in] size 消去エリアサイズ
@return なし
@attention start_addrおよびsizeはFLASHのページサイズの倍数でなければならない．
*///////////////////////////////////////////////////////////////////////////////
void erase_flash(uint32_t start_addr, uint32_t size)
{
	uint32_t addr;
	FLASH_Unlock();
	for(addr = start_addr; addr < start_addr + size; addr += FLASH_PAGE_SIZE){
		FLASH_ErasePage(addr);
	}
}

////////////////////////////////////////////////////////////////////////////////
/** @brief FLASHの指定したアドレスへ2バイト書き込む
@param[in] address 書き込み先のアドレス
@param[in] data 書き込む2バイト
@return なし
*///////////////////////////////////////////////////////////////////////////////
void write_2bytes_to_flash(uint32_t address, uint16_t data)
{
	FLASH_ProgramHalfWord(address, data);
}

////////////////////////////////////////////////////////////////////////////////
/** @brief FLASHの指定したアドレスへ4バイト書き込む
@param[in] address 書き込み先のアドレス
@param[in] data 書き込む4バイト
@return なし
*///////////////////////////////////////////////////////////////////////////////
void write_4bytes_to_flash(uint32_t address, uint32_t data)
{
	FLASH_ProgramWord(address, data);
}

